#pragma once
#include <string>
#include <iostream>
#include <algorithm>
#include <stdio.h>
#include <tchar.h>
#include "Command.h"
#include "EnumToString.h"

int GetInt(std::string print)
{
	std::cout << print;
	int id;
	std::string strId;
	std::getline(std::cin, strId);
	std::stringstream myStream(strId);
	myStream >> id;
	std::cin.clear();
	return id;
}
std::string GetString( std::string val )
{
	std::cout << val;
	std::string str;
	std::getline(std::cin, str);
	return str;
}

class DeviceCommand : public Command
{
public:
	void Execute(ICOMSessionPtr session , ICOMSessionManagerPtr sessionManager )
	{
		// declare enum variables used for commands
		DeviceAudioState valAudioState;
		DeviceAudioLinkState valAudioLinkState;
		DeviceAudioLocation devAudioLocation;
		DeviceBatteryLevel batteryLevel;
		DeviceHeadsetType headsetType;
		DeviceHeadsetState headsetState;
		COMVolumeLevel volumeLevel;
		COMToneLevel toneLevel;
		COMActiveCallRing activeCallRing;
		COMAudioLimit audioLimit;
		COMPowerLevel powerLevel;
		COMLineType lineType;
		DeviceSensorControl sensorControl;
		DSPTWAPeriod twaPeriod;
		COMRingTone ringTone;
		COMAudioBandwidth audioBandwidth;
		ICOMBatteryInfoPtr batteryInfo;
		VARIANT_BOOL valBool;
		VARIANT valVariant;
		BSTR valBstr = nullptr;
		SHORT valShort;
		HRESULT res = S_OK;
		

		{
			ICOMDevicePtr device;
			if( FAILED ( res = session->GetActiveDevice( &device ) ) )
			{
				std::cout << "Error gettting device interface. "; PRINT_STATUS ( res );
				return;
			}
			PrintDevice( device );
		}		

		do
		{
			std::cout << "IHostCommand(Ext) > ";
			// get user input
			std::string input;
			std::getline(std::cin, input);
			std::transform(input.begin(), input.end(), input.begin(), ::tolower);
			if( input == "exit" ) break;

			ICOMDevicePtr device;
			ICOMHostCommandPtr hostCommand;
			ICOMHostCommandExtPtr hostCommandExt;
			ICOMDeviceSettingsPtr deviceSettings;
			ICOMAdvanceSettingsPtr advanceSettings;
			ICOMATDCommandPtr atdCommand;

			if( SUCCEEDED ( res = session->GetActiveDevice( &device ) ) )
			{
				device->get_HostCommand( &hostCommand );
				hostCommandExt = hostCommand;
				deviceSettings = hostCommand;
				advanceSettings = hostCommand;
				atdCommand = hostCommand;
			}
			else
			{
				PRINT_ERROR( "Error gettting device interface. ", res );
				return;
			}	

			res = E_NOT_SET;
			if( valBstr ) ::SysFreeString( valBstr );
			if( hostCommand != nullptr )
			{
				if		( input == "ringon" )			res = hostCommand->put_SetRing( VARIANT_TRUE );
				else if ( input == "ringoff" )			res = hostCommand->put_SetRing( VARIANT_FALSE );
				else if ( input == "getaudiostate"){	res = hostCommand->get_AudioState( &valAudioState ); PrintReturn( valAudioState ); } 
				else if ( input == "putaudiostate")		res = hostCommand->put_AudioState( (DeviceAudioState) GetInt( "Enter DeviceAudioState enum:" ) );
				else if ( input == "getmute"){			res = hostCommand->get_mute( &valBool ); PrintReturn ( valBool ); }
				else if ( input == "getfwversion"){		res = hostCommand->GetFirmwareVersion( (FirmwareVersionType) GetInt( "Enter FirmwareVersionType enum:" ),  &valBstr);  PrintReturn( valBstr ); }
				else if ( input == "registercommand"){	res = hostCommand->RegisterCommand( (DeviceRegistrationType) GetInt( "Enter DeviceRegistrationType enum:")); }
				else if ( input == "help" )
				{
					std::cout << "ringon:             hostCommand->put_SetRing"		<< std::endl;
					std::cout << "ringoff:            hostCommand->put_SetRing"		<< std::endl;
					std::cout << "getaudiostate:      hostCommand->get_AudioState"	<< std::endl;
					std::cout << "putaudiostate:      hostCommand->put_AudioState"	<< std::endl;
					std::cout << "getfwversion:       hostCommand->GetFirmwareVersion"<< std::endl;
					std::cout << "registercommand:    hostCommand->RegisterCommand"	<< std::endl;
				}

				if( hostCommandExt != nullptr )
				{
					if		( input == "getbatterylevel" ){	 res = hostCommandExt->get_BatteryLevel( &batteryLevel );	PrintReturn( batteryLevel ); }
					else if ( input == "headsetinconf" ){    res = hostCommandExt->get_NumHeadsetInConf( &valShort );	PrintReturn( (LONG) valShort ); }
					else if ( input == "putheadsetmuteon" )  res = hostCommandExt->put_MuteHeadset( VARIANT_TRUE ); 
					else if ( input == "putheadsetmuteoff" ) res = hostCommandExt->put_MuteHeadset( VARIANT_FALSE ); 
					else if ( input == "getheadsettype" ){   res = hostCommandExt->get_HeadsetType( &headsetType );		PrintReturn( headsetType ); }
					else if ( input == "setactivelinkon"){	 res = hostCommandExt->SetActiveLink( (COMLineType )    GetInt( "Enter COMLineType enum" ), VARIANT_TRUE); }
					else if ( input == "setactivelinkoff"){	 res = hostCommandExt->SetActiveLink( (COMLineType )	   GetInt( "Enter COMLineType enum" ), VARIANT_FALSE); }
					else if ( input == "islineactive"){		 res = hostCommandExt->IsLineActive( (COMLineType )     GetInt( "Enter COMLineType enum" ), &valBool);  PrintReturn ( valBool ); } 
					else if ( input == "setlinkholdstateon") res = hostCommandExt->SetLinkHoldState( (COMLineType ) GetInt( "Enter COMLineType enum" ), VARIANT_TRUE ) ;
					else if ( input == "setlinkholdstateoff")res = hostCommandExt->SetLinkHoldState( (COMLineType ) GetInt( "Enter COMLineType enum" ), VARIANT_FALSE) ;
					else if ( input == "getlinkholdstate"){  res = hostCommandExt->GetLinkHoldState( (COMLineType ) GetInt( "Enter COMLineType enum" ), &valBool);  PrintReturn ( valBool ); } 
					else if ( input == "getaudiolinkstate"){ res = hostCommandExt->get_AudioLinkState( &valAudioLinkState);  PrintReturn ( valAudioLinkState ); } 
					else if ( input == "getheadsetdocked"){  res = hostCommandExt->get_HeadsetDocked( &valBool);  PrintReturn ( valBool ); } 
					else if ( input == "getheadprodname"){   res = hostCommandExt->get_HeadsetProductName( &valBstr);  PrintReturn ( valBstr ); } 
					else if ( input == "getaudiolocation"){  res = hostCommandExt->get_AudioLocation( &devAudioLocation);  PrintReturn (devAudioLocation ); } 
					else if ( input == "requestsernumber")   res = hostCommandExt->RequestSerialNumber( (COMDeviceType)GetInt( "Enter COMDeviceType enum" ) );
					else if ( input == "getserialnumber"){   res = hostCommandExt->GetSerialNumber((COMDeviceType)GetInt( "Enter COMDeviceType enum" ), &valVariant); PrintReturn ( valVariant ); }
					else if ( input == "requestproximity")   res = hostCommandExt->RequestProximity();
					else if ( input == "enableproximity")    res = hostCommandExt->EnableProximity(VARIANT_TRUE);
					else if ( input == "disableproximity")   res = hostCommandExt->EnableProximity(VARIANT_FALSE);
					else if ( input == "getheadsetstate"){   res = hostCommandExt->get_HeadsetState(&headsetState); PrintReturn (headsetState ); } 
					else if ( input == "getbatteryinfo"){    res = hostCommandExt->get_BatteryInfo( &batteryInfo); PrintReturn( batteryInfo ); }
					else if ( input == "help" )
					{
						std::cout << "======================================================="	 << std::endl;
						std::cout << "getbatterylevel:    hostCommandExt->get_BatteryLevel"		 << std::endl;
						std::cout << "headsetinconf:      hostCommandExt->get_NumHeadsetInConf"	 << std::endl;
						std::cout << "putheadsetmuteon:   hostCommandExt->put_MuteHeadset"		 << std::endl;
						std::cout << "putheadsetmuteoff:  hostCommandExt->put_MuteHeadset"		 << std::endl;
						std::cout << "getheadsettype:     hostCommandExt->get_HeadsetType"		 << std::endl;
						std::cout << "setactivelinkon:    hostCommandExt->SetActiveLink"		 << std::endl;
						std::cout << "setactivelinkoff:   hostCommandExt->SetActiveLink"		 << std::endl;
						std::cout << "islineactive:       hostCommandExt->IsLineActive"			 << std::endl;
						std::cout << "setlinkholdstateon: hostCommandExt->SetLinkHoldState"		 << std::endl;
						std::cout << "setlinkholdstateoff:hostCommandExt->SetLinkHoldState"		 << std::endl;
						std::cout << "getlinkholdstate:   hostCommandExt->GetLinkHoldState"		 << std::endl;
						std::cout << "getdialedkey:       hostCommandExt->get_DialedKey"		 << std::endl;
						std::cout << "getaudiolinkstate:  hostCommandExt->get_AudioLinkState"	 << std::endl;
						std::cout << "getheadsetdocked:   hostCommandExt->get_HeadsetDocked"	 << std::endl;
						std::cout << "getheadprodname:    hostCommandExt->get_HeadsetProductName"<< std::endl;
						std::cout << "getaudiolocation:   hostCommandExt->get_AudioLocation"	 << std::endl;
						std::cout << "requestsernumber:   hostCommandExt->RequestSerialNumber"	 << std::endl;
						std::cout << "requestproximity:   hostCommandExt->RequestProximity"		 << std::endl;
						std::cout << "enableproximity:    hostCommandExt->EnableProximity"		 << std::endl;
						std::cout << "disableproximity:   hostCommandExt->EnableProximity"		 << std::endl;
						std::cout << "getheadsetstate:    hostCommandExt->get_HeadsetState"		 << std::endl;
						std::cout << "getbatteryinfo:     hostCommandExt->get_BatteryInfo"		 << std::endl;
					}
				}

				if ( deviceSettings != nullptr )
				{
					if		( input == "getvoipring" ){	res = deviceSettings->get_VOIPRing(&ringTone); PrintReturn (ringTone); }
					else if ( input == "setvoipring") res = deviceSettings->put_VOIPRing( (COMRingTone) GetInt( "Enter COMRingTone enum: " ) );
					else if ( input == "getvoipbandwidth"){ res = deviceSettings->get_VOIPBandwidth( &audioBandwidth ); PrintReturn( audioBandwidth ); }
					else if ( input == "setvoipbandwidth") res = deviceSettings->put_VOIPBandwidth( (COMAudioBandwidth) GetInt( "Enter COMAudioBandwidth enum: " ) );
					else if ( input == "getvoipringvolume"){ res = deviceSettings->get_VOIPRingVolume( &volumeLevel ); PrintReturn( volumeLevel ) ; }
					else if ( input == "setvoipringvolume") res = deviceSettings->put_VOIPRingVolume( (COMVolumeLevel) GetInt( "Enter COMVolumeLevel enum: " ) );
					else if ( input == "getvoiptonelevel") { res = deviceSettings->get_VOIPToneLevel( &toneLevel ); PrintReturn( toneLevel ) ;}
					else if ( input == "setvoiptonelevel") res = deviceSettings->put_VOIPToneLevel( (COMToneLevel) GetInt( "Enter COMToneLevel enum: " ) );
					else if ( input == "getpstnring") { res = deviceSettings->get_PSTNRing( &ringTone ); PrintReturn( ringTone ); }
					else if ( input == "setpstnring") res = deviceSettings->put_PSTNRing( (COMRingTone) GetInt( "Enter COMRingTone enum" ) );
					else if ( input == "getpstnbandwidth") { res = deviceSettings->get_PSTNBandwidth( &audioBandwidth ) ; PrintReturn( audioBandwidth ); }
					else if ( input == "setpstnbandwidth") res = deviceSettings->put_PSTNBandwidth( (COMAudioBandwidth) GetInt( "Enter COMAudioBandwidth enum: " ));
					else if ( input == "getpstnringvolume") { res = deviceSettings->get_PSTNRingVolume( &volumeLevel ); PrintReturn( volumeLevel ); }
					else if ( input == "setpstnringvolume") res = deviceSettings->put_PSTNRingVolume( (COMVolumeLevel) GetInt( "Enter COMVolumeLevel enum: " ) );
					else if ( input == "getpstntonelevel") { res = deviceSettings->get_PSTNToneLevel( &toneLevel); PrintReturn( toneLevel ); }
					else if ( input == "setpstntonelevel") res = deviceSettings->put_PSTNToneLevel( (COMToneLevel) GetInt( "Enter COMToneLevel enum: " ) );
					else if ( input == "getmobilering") { res = deviceSettings->get_MobileRing( &ringTone ); PrintReturn( ringTone ) ; }
					else if ( input == "setmobilering") res = deviceSettings->put_MobileRing( (COMRingTone) GetInt( "Enter COMRingTone enum" ) );
					else if ( input == "getmobileringvolume") { res = deviceSettings->get_MobileRingVolume( &volumeLevel ); PrintReturn( volumeLevel );}
					else if ( input == "setmobileringvolume") res = deviceSettings->put_MobileRingVolume( (COMVolumeLevel) GetInt("Enter COMVolumeLevel enum: " ) );
					else if ( input == "getmobilebandwidth") { res = deviceSettings->get_MobileBandwidth( &audioBandwidth ) ; PrintReturn( audioBandwidth ); }
					else if ( input == "setmobilebandwidth") res = deviceSettings->put_MobileBandwidth( (COMAudioBandwidth) GetInt("Enter COMAudioBandwidth enum: " ) );
					else if ( input == "getmutetone") { res = deviceSettings->get_MuteTone( &ringTone ); PrintReturn( ringTone ) ; }
					else if ( input == "setmutetone") res = deviceSettings->put_MuteTone( (COMRingTone) GetInt("Enter COMRingTone enum: " ) );
					else if ( input == "gettonevolume") { res = deviceSettings->get_ToneVolume( &volumeLevel ); PrintReturn( volumeLevel); }
					else if ( input == "settonevolume") res = deviceSettings->put_ToneVolume( (COMVolumeLevel) GetInt( "Enter COMVolumeLevel enum: " ) );
					else if ( input == "getmutetonevolume" ) { res = deviceSettings->get_MuteToneVolume( &volumeLevel ); PrintReturn( volumeLevel); }
					else if ( input == "setmutetonevolume" ) res = deviceSettings->put_MuteToneVolume( (COMVolumeLevel) GetInt( "Enter COMVolumeLevel enum: " ) );
					else if ( input == "getactivecallring" ) { res = deviceSettings->get_ActiveCallRing( &activeCallRing); PrintReturn( activeCallRing ); }
					else if ( input == "setactivecallring" ) res = deviceSettings->put_ActiveCallRing( (COMActiveCallRing) GetInt( "Enter COMActiveCallRing enum: " ) );
					else if ( input == "getantistartleenabled" ) { res = deviceSettings->get_AntiStartleEnabled( &valBool) ; PrintReturn( valBool ); }
					else if ( input == "setantistartleenabledon") res = deviceSettings->put_AntiStartleEnabled( VARIANT_TRUE );
					else if ( input == "setantistartleenabledoff") res = deviceSettings->put_AntiStartleEnabled( VARIANT_FALSE );
					else if ( input == "getaudiolimit") { res = deviceSettings->get_AudioLimit( &audioLimit); PrintReturn(audioLimit ); }
					else if ( input == "setaudiolimit") res = deviceSettings->put_AudioLimit( ( COMAudioLimit ) GetInt("Enter DeviceAucioLimit enum: " ) );
					else if ( input == "getg616enabled") { res = deviceSettings->get_G616Enabled( &valBool ); PrintReturn( valBool ); }
					else if ( input == "setg616enabledon") res = deviceSettings->put_G616Enabled( VARIANT_TRUE );
					else if ( input == "setg616enabledoff") res = deviceSettings->put_G616Enabled( VARIANT_FALSE );
					else if ( input == "getotaenabled") { res = deviceSettings->get_OTAEnabled( &valBool ); PrintReturn( valBool ); }
					else if ( input == "setotaenabledon") res = deviceSettings->put_OTAEnabled( VARIANT_TRUE );
					else if ( input == "setotaenabledoff") res = deviceSettings->put_OTAEnabled( VARIANT_FALSE );
					else if ( input == "getintellistandenabled") { res = deviceSettings->get_IntellistandEnabled( &valBool ); PrintReturn( valBool ); }
					else if ( input == "setintellistandenabledon") res = deviceSettings->put_IntellistandEnabled( VARIANT_TRUE );
					else if ( input == "setintellistandenabledoff") res = deviceSettings->put_IntellistandEnabled( VARIANT_FALSE );
					else if ( input == "getpowermode" ) { res = deviceSettings->get_PowerMode( &powerLevel ); PrintReturn( powerLevel ); }
					else if ( input == "setpowermode" ) res = deviceSettings->put_PowerMode( (COMPowerLevel) GetInt("Enter DevicePowerLevel enum: " ) ) ;
					else if ( input == "getpasswordprotected") { res = deviceSettings->get_PasswordProtected( &valBool ); PrintReturn( valBool ); }
					else if ( input == "getdectmode" ) { res = deviceSettings->get_DECTMode( &valBool ); PrintReturn( valBool ) ; }
					else if ( input == "gettwaperiod") { res = deviceSettings->get_TWAPeriod( &twaPeriod ) ; PrintReturn(twaPeriod ); }
					else if ( input == "settwaperiod") res = deviceSettings->put_TWAPeriod( (DSPTWAPeriod) GetInt("Enter DSPTWAPeriod enum: " ) ); 
					else if ( input == "getphoneline") { res = deviceSettings->get_PhoneLine( &lineType) ; PrintReturn( lineType ); }
					else if ( input == "setphoneline") res = deviceSettings->put_PhoneLine( (COMLineType) GetInt("Enter COMLineType enum: " ) );
					else if ( input == "getbtinterfaceenabled" ) { res = deviceSettings->get_BTInterfaceEnabled( &valBool ); PrintReturn( valBool ); }
					else if ( input == "setbtinterfaceenabledon" ) res = deviceSettings->put_BTInterfaceEnabled( VARIANT_TRUE );
					else if ( input == "setbtinterfaceenabledoff" ) res = deviceSettings->put_BTInterfaceEnabled( VARIANT_FALSE );
					else if ( input == "getbtautoconnectenabled" ) { res = deviceSettings->get_BTAutoConnectEnabled( &valBool ); PrintReturn( valBool ); }
					else if ( input == "setbtautoconnectenabledon" ) res = deviceSettings->put_BTAutoConnectEnabled( VARIANT_TRUE );
					else if ( input == "setbtautoconnectenabledoff" ) res = deviceSettings->put_BTAutoConnectEnabled( VARIANT_FALSE );
					else if ( input == "restoredefaults" ) res = deviceSettings->RestoreDefaultSettings();
					else if ( input == "getaclpollingenabled" ) { res = deviceSettings->get_ACLPollingEnabled( &valBool ) ; PrintReturn( valBool ); }
					else if ( input == "setaclpollingenabledon" ) res = deviceSettings->put_ACLPollingEnabled( VARIANT_TRUE );
					else if ( input == "setaclpollingenabledoff" ) res = deviceSettings->put_ACLPollingEnabled( VARIANT_FALSE );
					else if ( input == "getbtvoicecommandenabled" ) { res = deviceSettings->get_BTVoiceCommandEnabled( &valBool ) ; PrintReturn( valBool ) ; }
					else if ( input == "setbtvoicecommandenabledon" ) res = deviceSettings->put_BTVoiceCommandEnabled( VARIANT_TRUE );
					else if ( input == "setbtvoicecommandenabledoff" ) res = deviceSettings->put_BTVoiceCommandEnabled( VARIANT_FALSE );
					else if ( input == "setdata" ) 
					{
						std::string command = GetString("Enter string you would like to write to device using SetData()> ");
						res = SendDataBSTR(deviceSettings, Custom_Data_Raw_HID_Pipe_Data, CComBSTR(command.c_str()).Detach());
					}
					else if ( input == "help" )
					{
						std::cout << "======================================================="	 << std::endl;
						std::cout << "getvoipring:               deviceSettings->get_VOIPRing" << std::endl;
						std::cout << "setvoipring:               deviceSettings->put_VOIPRing" << std::endl;
						std::cout << "getvoipbandwidth:          deviceSettings->get_VOIPBandwidth" << std::endl;
						std::cout << "setvoipbandwidth:          deviceSettings->put_VOIPBandwidth" << std::endl;
						std::cout << "getvoipringvolume:         deviceSettings->get_VOIPRingVolume" << std::endl;
						std::cout << "setvoipringvolume:         deviceSettings->put_VOIPRingVolume" << std::endl;
						std::cout << "getvoiptonelevel:          deviceSettings->get_VOIPToneLevel" << std::endl;
						std::cout << "setvoiptonelevel:          deviceSettings->put_VOIPToneLevel" << std::endl;
						std::cout << "getpstnring:               deviceSettings->get_PSTNRing" << std::endl;
						std::cout << "setpstnring:               deviceSettings->put_PSTNRing" << std::endl;
						std::cout << "getpstnbandwidth:          deviceSettings->get_PSTNBandwidth" << std::endl;	
						std::cout << "setpstnbandwidth:          deviceSettings->put_PSTNBandwidth" << std::endl;		
						std::cout << "getpstnringvolume:         deviceSettings->get_PSTNRingVolume" << std::endl;
						std::cout << "setpstnringvolume:         deviceSettings->put_PSTNRingVolume" << std::endl;
						std::cout << "getpstntonelevel:          deviceSettings->get_PSTNToneLevel" << std::endl;
						std::cout << "setpstntoneleve:           deviceSettings->put_PSTNToneLevel" << std::endl;
						std::cout << "getmobilering:             deviceSettings->get_MobileRing" << std::endl;
						std::cout << "setmobilering:             deviceSettings->put_MobileRing" << std::endl;
						std::cout << "getmobileringvolume:       deviceSettings->get_MobileRingVolume" << std::endl;
						std::cout << "setmobileringvolume:       deviceSettings->put_MobileRingVolume" << std::endl;
						std::cout << "getmobilebandwidth:        deviceSettings->get_MobileBandwidth" << std::endl;
						std::cout << "setmobilebandwidth:        deviceSettings->put_MobileBandwidth" << std::endl;
						std::cout << "getmutetone:               deviceSettings->get_MuteTone" << std::endl;
						std::cout << "setmutetone:               deviceSettings->put_MuteTone" << std::endl;
						std::cout << "gettonevolume:             deviceSettings->get_ToneVolume" << std::endl;
						std::cout << "settonevolume:             deviceSettings->put_ToneVolume" << std::endl;
						std::cout << "getmutetonevolume:         deviceSettings->get_MuteToneVolume" << std::endl;
						std::cout << "setmutetonevolume:         deviceSettings->put_MuteToneVolume" << std::endl;
						std::cout << "getactivecallring:         deviceSettings->get_ActiveCallRing" << std::endl;
						std::cout << "setactivecallring:         deviceSettings->put_ActiveCallRing" << std::endl;
						std::cout << "getantistartleenabled:     deviceSettings->get_AntiStartleEnabled" << std::endl;
						std::cout << "setantistartleenabledon:   deviceSettings->put_AntiStartleEnabled" << std::endl;
						std::cout << "setantistartleenabledoff:  deviceSettings->put_AntiStartleEnabled" << std::endl;
						std::cout << "getaudiolimit:             deviceSettings->get_AudioLimit" << std::endl;
						std::cout << "setaudiolimit:             deviceSettings->put_AudioLimit" << std::endl;
						std::cout << "getg616enabled:            deviceSettings->get_G616Enabled" << std::endl;
						std::cout << "setg616enabledon:          deviceSettings->put_G616Enabled" << std::endl;
						std::cout << "setg616enabledoff:         deviceSettings->put_G616Enabled" << std::endl;
						std::cout << "getotaenabled:             deviceSettings->get_OTAEnabled" << std::endl;
						std::cout << "setotaenabledon:           deviceSettings->put_OTAEnabled" << std::endl;
						std::cout << "setotaenabledoff:          deviceSettings->put_OTAEnabled" << std::endl;
						std::cout << "getintellistandenabled:    deviceSettings->get_IntellistandEnabled" << std::endl;
						std::cout << "setintellistandenabledon:  deviceSettings->put_IntellistandEnabled" << std::endl;
						std::cout << "setintellistandenabledoff: deviceSettings->put_IntellistandEnabled" << std::endl;
						std::cout << "getpowermode:              deviceSettings->get_PowerMode" << std::endl;
						std::cout << "setpowermode:              deviceSettings->put_PowerMode" << std::endl;
						std::cout << "getpasswordprotected:      deviceSettings->get_PasswordProtected" << std::endl;
						std::cout << "getdectmode:               deviceSettings->get_DECTMode" << std::endl;
						std::cout << "gettwaperiod:              deviceSettings->get_TWAPeriod" << std::endl;
						std::cout << "settwaperiod:              deviceSettings->put_TWAPeriod" << std::endl;
						std::cout << "getphoneline:              deviceSettings->get_PhoneLine" << std::endl;
						std::cout << "setphoneline:              deviceSettings->put_PhoneLine" << std::endl;
						std::cout << "getbtinterfaceenabled:     deviceSettings->get_BTInterfaceEnabled" << std::endl;
						std::cout << "setbtinterfaceenabledon:   deviceSettings->put_BTInterfaceEnabled" << std::endl;
						std::cout << "setbtinterfaceenabledoff:  deviceSettings->put_BTInterfaceEnabled" << std::endl;
						std::cout << "getbtautoconnectenabled:   deviceSettings->get_BTAutoConnectEnabled" << std::endl;
						std::cout << "setbtautoconnectenabledon: deviceSettings->put_BTAutoConnectEnabled" << std::endl;
						std::cout << "setbtautoconnectenabledoff:deviceSettings->put_BTAutoConnectEnabled" << std::endl;
						std::cout << "restoredefaults:           deviceSettings->RestoreDefaultSettings" << std::endl;
						std::cout << "getaclpollingenabled:      deviceSettings->get_ACLPollingEnabled" << std::endl;
						std::cout << "setaclpollingenabledon:    deviceSettings->put_ACLPollingEnabled" << std::endl;
						std::cout << "setaclpollingenabledoff:   deviceSettings->put_ACLPollingEnabled" << std::endl;
						std::cout << "getbtvoicecommandenabled:  deviceSettings->get_BTVoiceCommandEnabled" << std::endl;
						std::cout << "setbtvoicecommandenabledon:deviceSettings->put_BTVoiceCommandEnabled" << std::endl;
						std::cout << "setbtvoicecommandenabledoff:deviceSettings->put_BTVoiceCommandEnabled" << std::endl;
						std::cout << "======================================================="	 << std::endl;
					}
				}

				if( advanceSettings != nullptr )
				{
					if		( input == "getdialtone" )		{ res = advanceSettings->get_DialTone( &valBool);			  PrintReturn( valBool ); }
					else if ( input == "getaudiosensing" )	{ res = advanceSettings->get_AudioSensing( &valBool);		  PrintReturn( valBool ); }
					else if ( input == "getdialtoneactive" ){ res = advanceSettings->get_DialToneActive( &valBool);		  PrintReturn( valBool ); }
					else if ( input == "getanswerondon" )	{ res = advanceSettings->get_AnswerOnDon( &valBool);		  PrintReturn( valBool ); }
					else if ( input == "getautotransfer" )	{ res = advanceSettings->get_AutoTransfer( &sensorControl );  PrintReturn( sensorControl ); }
					else if ( input == "getautodisconnect" ){ res = advanceSettings->get_AutoDisconnect( &sensorControl); PrintReturn( sensorControl ); }
					else if ( input == "getautoreject" )	{ res = advanceSettings->get_AutoReject( &sensorControl ) ;	  PrintReturn( sensorControl ); }
					else if ( input == "getlockhookswitch" ){ res = advanceSettings->get_LockHookSwitch( &sensorControl );PrintReturn( sensorControl ); }
					else if ( input == "getautopause" )		{ res = advanceSettings->get_AutoPause( &sensorControl ) ;	  PrintReturn( sensorControl ); }
					else if ( input == "getvoiceprompt" )	{ res = advanceSettings->get_VoicePrompt( &sensorControl ) ;  PrintReturn( sensorControl ); }
					else if ( input == "setdialtoneon")		  res = advanceSettings->put_DialTone( VARIANT_TRUE );
					else if ( input == "setdialtoneoff")	  res = advanceSettings->put_DialTone( VARIANT_FALSE );
					else if ( input == "setaudiosensingon")   res = advanceSettings->put_AudioSensing( VARIANT_TRUE );
					else if ( input == "setaudiosensingoff")  res = advanceSettings->put_AudioSensing( VARIANT_FALSE );
					else if ( input == "setanswerondonon")	  res = advanceSettings->put_AnswerOnDon( VARIANT_TRUE );
					else if ( input == "setanswerondonoff")   res = advanceSettings->put_AnswerOnDon( VARIANT_FALSE );
					else if ( input == "setautotransfer" )	  res = advanceSettings->put_AutoTransfer( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "setautodisconnect" )  res = advanceSettings->put_AutoDisconnect( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "setautoreject" )	  res = advanceSettings->put_AutoReject( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "setlockhookswitch" )  res = advanceSettings->put_LockHookSwitch( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "setautopause" )		  res = advanceSettings->put_AutoPause( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "setvoiceprompt" )	  res = advanceSettings->put_VoicePrompt( (DeviceSensorControl) GetInt("Enter DeviceSensorControl enum: " ) );
					else if ( input == "help")
					{
						std::cout << "======================================================="	<< std::endl;
						std::cout << "getdialtone:          advanceSettings->get_DialTone"		<< std::endl;			  
						std::cout << "getaudiosensing:      advanceSettings->get_AudioSensing"	<< std::endl;		  
						std::cout << "getdialtoneactive:    advanceSettings->get_DialToneActive"<< std::endl;	  
						std::cout << "getanswerondon:       advanceSettings->get_AnswerOnDon"	<< std::endl;		  
						std::cout << "getautotransfer:      advanceSettings->get_AutoTransfer"	<< std::endl;
						std::cout << "getautodisconnect:    advanceSettings->get_AutoDisconnect"<< std::endl;
						std::cout << "getautoreject:        advanceSettings->get_AutoReject"	<< std::endl;
						std::cout << "getlockhookswitch:    advanceSettings->get_LockHookSwitch"<< std::endl;
						std::cout << "getautopause:         advanceSettings->get_AutoPause"		<< std::endl;	  
						std::cout << "getvoiceprompt:       advanceSettings->get_VoicePrompt"	<< std::endl;
						std::cout << "setdialtoneon:        advanceSettings->put_DialTone"		<< std::endl;
						std::cout << "setdialtoneoff:       advanceSettings->put_DialTone"		<< std::endl;
						std::cout << "setaudiosensingon:    advanceSettings->put_AudioSensing"	<< std::endl;
						std::cout << "setaudiosensingoff:   advanceSettings->put_AudioSensing"	<< std::endl;
						std::cout << "setanswerondonon:     advanceSettings->put_AnswerOnDon"	<< std::endl;
						std::cout << "setanswerondonoff:    advanceSettings->put_AnswerOnDon"	<< std::endl;
						std::cout << "setautotransfer:      advanceSettings->put_AutoTransfer"	<< std::endl;
						std::cout << "setautodisconnect:    advanceSettings->put_AutoDisconnect"<< std::endl;
						std::cout << "setautoreject:        advanceSettings->put_AutoReject"	<< std::endl;
						std::cout << "setlockhookswitch:    advanceSettings->put_LockHookSwitch"<< std::endl;
						std::cout << "setautopause:         advanceSettings->put_AutoPause"		<< std::endl;
						std::cout << "setvoiceprompt:       advanceSettings->put_VoicePrompt"	<< std::endl;
						std::cout << "======================================================="	<< std::endl;
					}
				}
				
				if( atdCommand != nullptr )
				{
					if      ( input == "answermobilecall" )			res = atdCommand->AnswerMobileCall();
					else if ( input == "endmobilecall" )			res = atdCommand->EndMobileCall();
					else if ( input == "redial" )					res = atdCommand->Redial();
					else if ( input == "makemobilecall" )			res = atdCommand->MakeMobileCall( valBstr = _bstr_t(  GetString("Enter caller Id: " ).c_str() ).Detach() );
					else if ( input == "requestmobilecallstatus" )	res = atdCommand->RequestMobileCallStatus();
					else if ( input == "mutemobilecallon")			res = atdCommand->MuteMobileCall( VARIANT_TRUE );
					else if ( input == "mutemobilecalloff")			res = atdCommand->MuteMobileCall( VARIANT_FALSE );
					//else if ( input == "getcallerid" ){				res = atdCommand->GetCallerId( &valBstr ); PrintReturn( valBstr ); }
					else if ( input == "help" )
					{
						std::cout << "answermobilecall:       atdCommand->AnswerMobileCall"	    << std::endl;
						std::cout << "endmobilecall:          atdCommand->EndMobileCall"	    << std::endl;
						std::cout << "redial:                 atdCommand->Redial"	            << std::endl;
						std::cout << "makemobilecall:         atdCommand->MakeMobileCall"	    << std::endl;
						std::cout << "requestmobilecallstatus:atdCommand->RequestMobileCallStatus"<< std::endl;
						std::cout << "mutemobilecallon:       atdCommand->MuteMobileCall"	    << std::endl;
						std::cout << "mutemobilecalloff:      atdCommand->MuteMobileCall"	    << std::endl;
						std::cout << "getcallerid:            atdCommand->get_callerId"	        << std::endl;
						std::cout << "======================================================="	<< std::endl;
					}
				}
			}
			
			if(res == E_NOT_SET) { std::cout << "type help for list of commands" << std::endl; continue; }
			
			PRINT_STATUS( res );
			
		} while (true);

		if( valBstr ) ::SysFreeString( valBstr );
		delete this;
	}
private:
	void PrintDevice( ICOMDevicePtr pDevice )
	{
		if( pDevice != nullptr )
		{
			BSTR internalName, manufacturerName, productName, serialNumber;
			
			USHORT productId, vendorId, versionNum;
			pDevice->get_InternalName( &internalName );
			pDevice->get_ManufacturerName( &manufacturerName );
			pDevice->get_ProductId( &productId );
			pDevice->get_ProductName( &productName );
			pDevice->get_SerialNumber( &serialNumber );
			pDevice->get_VendorId( &vendorId );
			pDevice->get_VersionNumber( &versionNum );

			_bstr_t binternalName(internalName), bmanufacturerName(manufacturerName), bproductName(productName), bserialNumber(serialNumber);
			std::wcout << L"************************************************************" << std::endl;
			std::wcout << L"* Internal Name:     " << binternalName		<< std::endl;
			std::wcout << L"* Manufacturer Name: " << bmanufacturerName << std::endl;
			std::wcout << L"* Product Name:	     " << bproductName		<< std::endl;
			std::wcout << L"* Serial no.:        " << bserialNumber		<< std::endl;
			std::wcout << L"* Product ID:        " << L"0x" << std::hex << std::setfill(L'0') << std::setw(4) << productId << std::endl;
			std::wcout << L"* Vendor ID:         " << L"0x" << std::hex << vendorId << std::endl;
			std::wcout << L"* Version no.:       " << L"0x" << std::hex << versionNum << std::endl;
			std::wcout << L"************************************************************" << std::endl;
		}
	}
	
	HRESULT SendDataBSTR(ICOMDeviceSettingsPtr pDeviceSettings, COMCustomData dataType, BSTR str)
	{
		LONG len = SysStringLen(str);

		SAFEARRAY* psarray;
		SAFEARRAYBOUND sabounds[1];	

		sabounds[0].lLbound=0;
		sabounds[0].cElements = (ULONG)len;

		psarray = SafeArrayCreate(VT_UI1, 1, sabounds);

		LONG i;
		//converting string to bytes and adding them to SAFEARRAY
		for (i=0; i<len; i++)
		{
			BYTE b = (BYTE) str[i];
			SafeArrayPutElement(psarray, &i, &b);
		}
		return pDeviceSettings->SetData(dataType, psarray);
	}
};
