#pragma once
#include <string>
#include <iostream>
#include <algorithm>
#include <stdio.h>
#include <tchar.h>
#include "Command.h"

class DeviceListener : public Command
{
public:

	void Execute(ICOMSessionPtr session , ICOMSessionManagerPtr sessionManager )
	{
		DeviceAudioState deviceAudioState;
		LONG deviceMuteState;
		COMLineType comLineType;
		DeviceAudioLocation devAudioLocation;
		DeviceAudioLinkState valAudioLinkState;
		VARIANT_BOOL valBool;
		SHORT valInt;

		HRESULT res = S_OK;
		
		do
		{
			std::cout << "IDeviceListener > ";
			// get user input
			std::string input;
			std::getline(std::cin, input);
			std::transform(input.begin(), input.end(), input.begin(), ::tolower);
			if( input == "exit" ) break;

			ICOMDevicePtr device;
			ICOMDeviceListenerPtr deviceListener;
			if( SUCCEEDED ( res = session->GetActiveDevice( &device ) ) ) // get current active device
				res = device->get_DeviceListener( &deviceListener ); // get device listener interface

			if( FAILED( res ) ) { PRINT_ERROR ( "Device is not attached", res ); return; }

			// execute command
			if		( input == "ringon" )			 res = deviceListener->Ring( VARIANT_TRUE );
			else if ( input == "ringoff")			 res = deviceListener->Ring( VARIANT_FALSE);
			else if ( input == "getaudiostate"){	 res = deviceListener->get_AudioState( &deviceAudioState ); PrintReturn( deviceAudioState ); }
			else if ( input == "putaudiostate")		 res = deviceListener->put_AudioState( (DeviceAudioState)GetInt( "Enter DeviceAudioState enum") );
			else if ( input == "getmute"){			 res = deviceListener->get_mute( &valBool ); PrintReturn( valBool ); }
			else if ( input == "putmuteon")			 res = deviceListener->put_mute( VARIANT_TRUE);
			else if ( input == "putmuteoff")		 res = deviceListener->put_mute( VARIANT_FALSE);
			else if ( input == "getintellistand"){	 res = deviceListener->get_Intellistand( &valBool ); PrintReturn( valBool );}
			else if ( input == "putintellistandon")  res = deviceListener->put_Intellistand( VARIANT_TRUE );
			else if ( input == "putintellistandoff") res = deviceListener->put_Intellistand( VARIANT_FALSE );
			else if ( input == "getaudiomixerstate"){res = deviceListener->get_AudioMixerState( &deviceMuteState ); PrintAudioMixerState( deviceMuteState ); }
			else if ( input == "putaudiomixerstate") res = deviceListener->put_AudioMixerState( GetInt( "Enter DeviceMuteState flag combination" ) );
			else if ( input == "getdefaultline"){	 res = deviceListener->get_DefaultLine( &comLineType); PrintReturn( comLineType ); }
			else if ( input == "putdefaultline")	 res = deviceListener->put_DefaultLine( (COMLineType) GetInt( "Enter COMLineType enum" ) );
			else if ( input == "incallon") 			 res = deviceListener->InCall( VARIANT_TRUE );
			else if ( input == "incalloff") 		 res = deviceListener->InCall( VARIANT_FALSE );
			else if ( input == "setactivelinkon")	 res = deviceListener->SetActiveLink(  (COMLineType) GetInt( "Enter COMLineType enum" ) , VARIANT_TRUE);
			else if ( input == "setactivelinkoff")	 res = deviceListener->SetActiveLink(  (COMLineType) GetInt( "Enter COMLineType enum" ) , VARIANT_FALSE);
			else if ( input == "islineactive"){		 res = deviceListener->IsLineActive(  (COMLineType) GetInt( "Enter COMLineType enum" ) , &valBool ); PrintReturn( valBool ); }
			else if ( input == "holdon")			 res = deviceListener->Hold( (COMLineType) GetInt( "Enter COMLineType enum" ), VARIANT_TRUE );
			else if ( input == "holdoff")			 res = deviceListener->Hold( (COMLineType) GetInt( "Enter COMLineType enum" ), VARIANT_FALSE );
			else if ( input == "getholdstate"){		 res = deviceListener->IsLineActive(  (COMLineType) GetInt( "Enter COMLineType enum" ) , &valBool ); PrintReturn( valBool ); }
			else if ( input == "getaudiolinkstate"){ res = deviceListener->get_AudioLinkState( &valAudioLinkState);  PrintReturn (valAudioLinkState ); } 
			else if ( input == "getaudiolocation"){	 res = deviceListener->get_AudioLocation( &devAudioLocation);  PrintReturn ( devAudioLocation ); } 
			// print help
			else if ( input == "help" )
			{
				std::cout << "ringon:				deviceListener->Ring(VARIANT_TRUE)"	<< std::endl;
				std::cout << "ringoff:				deviceListener->Ring(VARIANT_FALSE)" << std::endl;
				std::cout << "getaudiostate:		deviceListener->get_AudioState"	<< std::endl;
				std::cout << "putaudiostate:		deviceListener->put_AudioState"	<< std::endl;
				std::cout << "getmute:				deviceListener->get_mute"<< std::endl;
				std::cout << "putmuteon:			deviceListener->put_mute"	<< std::endl;
				std::cout << "putmuteoff:			deviceListener->put_mute"	<< std::endl;
				std::cout << "getintellistand:		deviceListener->get_Intellistand"	<< std::endl;
				std::cout << "putintellistandon:	deviceListener->put_Intellistand"	<< std::endl;
				std::cout << "putintellistandoff:	deviceListener->put_Intellistand"	<< std::endl;
				std::cout << "getaudiomixerstate:	deviceListener->get_AudioMixerState"	<< std::endl;
				std::cout << "putaudiomixerstate:	deviceListener->put_AudioMixerState"	<< std::endl;
				std::cout << "getdefaultline:		deviceListener->get_DefaultLine"	<< std::endl;
				std::cout << "putdefaultline:		deviceListener->put_DefaultLine"	<< std::endl;
				std::cout << "incallon:				deviceListener->InCall"	<< std::endl;
				std::cout << "incalloff:			deviceListener->InCall"	<< std::endl;
				std::cout << "getdialedkey:			deviceListener->get_DialedKey"	<< std::endl;
				std::cout << "setactivelinkon:		deviceListener->SetActiveLink"	<< std::endl;
				std::cout << "setactivelinkoff:		deviceListener->SetActiveLink"	<< std::endl;
				std::cout << "islineactive:			deviceListener->IsLineActive"	<< std::endl;
				std::cout << "holdon:				deviceListener->Hold"	<< std::endl;
				std::cout << "holdoff:				deviceListener->Hold"	<< std::endl;
				std::cout << "getholdstate:			deviceListener->IsLineActive"	<< std::endl;
				std::cout << "getaudiolinkstate:	deviceListener->get_AudioLinkState"	<< std::endl;
				std::cout << "getaudiolocation:		deviceListener->get_AudioLocation"	<< std::endl;
				continue;
			}
			else { std::cout << "type help for list of commands" << std::endl; continue; }
			// print return status
			PRINT_STATUS( res );

		} while (true);


		delete this;
	}
};
