#include "StdAfx.h"
#include "EventSink.h"
#include <iostream>
#include "EnumToString.h"

DeviceSink* gDeviceSink = nullptr;

_ATL_FUNC_INFO OnDeviceEvents = { CC_STDCALL, VT_EMPTY, 1, { VT_DISPATCH }};

void PrintCOMCallEventArgs(std::string str, struct ICOMCallEventArgs *callEventArgs )
{
	ICOMCall* call = nullptr;
	CallState callState;
	BSTR source;
	BSTR callerID;
	long callId(0);

	callEventArgs->get_CallSource(&source);
	callEventArgs->get_CallState(&callState);
	callEventArgs->get_CallerIdentity(&callerID);
	if( SUCCEEDED( callEventArgs->get_call(&call) ) && call != nullptr)
		call->get_Id(&callId);

	std::cout << std::endl << str << "CallState: " << EnumToString(callState ) << " Source: " << std::string(_bstr_t(source)) << " CallerID: " << std::string(_bstr_t(callerID)); 

	_bstr_t dtmfKey("DTMFKey"), contactPhone("ContactPhone"), contactFN( "ContactFriendlyName"), lineType( "LineType"), direction("Direction");
	BSTR dtmfKeyVal, contactPhoneVal, contactFNVal, lineTypeVal, directionVal;

	if( SUCCEEDED( callEventArgs->GetOptions(dtmfKey, &dtmfKeyVal ) ) && dtmfKeyVal != nullptr)					std::wcout << L" " << dtmfKey << ":" << dtmfKeyVal;
	if( SUCCEEDED( callEventArgs->GetOptions(contactPhone, &contactPhoneVal ) ) && contactPhoneVal != nullptr)	std::wcout << L" " << contactPhone << ":" << contactPhoneVal;
	if( SUCCEEDED( callEventArgs->GetOptions(contactFN, &contactFNVal ) )  && contactFNVal != nullptr)			std::wcout << L" " << contactFN << ":" << contactFNVal;
	if( SUCCEEDED( callEventArgs->GetOptions(lineType, &lineTypeVal ) )  && lineTypeVal != nullptr)				std::wcout << L" " << lineType << ":" << lineTypeVal;
	if( SUCCEEDED( callEventArgs->GetOptions(direction, &directionVal ) )  && directionVal != nullptr)			std::wcout << L" " << direction << ":" << directionVal;
	
}
STDMETHODIMP SessionEventSink::onCallRequested( struct ICOMCallRequestEventArgs *requestArgs)			
{
	std::string contactName;
	ICOMContact* callContact = nullptr;
	requestArgs->get_contact(&callContact);
	if( callContact != nullptr )
	{
		BSTR name;
		callContact->get_Name(&name);
		contactName = (name != nullptr ? _bstr_t( name ) : "none");
	}
	std::cout << std::endl << "Session onCallRequested contact: " << contactName ;
	return S_OK;
}
STDMETHODIMP SessionEventSink::onCallStateChanged( struct ICOMCallEventArgs *callEventArgs)			
{
	PrintCOMCallEventArgs( "Session onCallStateChanged ", callEventArgs);
	return S_OK;
}
STDMETHODIMP ManagerEventSink::onDeviceStateChanged( struct ICOMStateDeviceEventArgs *args)			
{
	DeviceChangeState devState;
	args->get_DeviceState(&devState);
	std::cout << std::endl << "Manager onDeviceStateChanged DeviceChangeState: " << EnumToString( devState );

	return S_OK;
}
STDMETHODIMP ManagerEventSink::onCallStateChanged(struct ICOMCallEventArgs *callEventArgs)		
{
	PrintCOMCallEventArgs( "Manager onCallStateChanged ", callEventArgs);
	return S_OK;
}

STDMETHODIMP DeviceEventSink::onDataReceived(VARIANT *report)
{
	// uncomment the following line to trace reports received from device
	//std::cout << std::endl << "ICOMDeviceEvents onDataReceived " << VariantToString(*report);
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onTalkButtonPressed(struct ICOMDeviceEventArgs *args)
{
	std::cout << std::endl << "ICOMDeviceEvents onTalkButtonPressed ";
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onButtonPressed(struct ICOMDeviceEventArgs *args)
{
	DeviceHeadsetButton button;
	args->get_ButtonPressed(&button);
	std::cout << std::endl << "ICOMDeviceEvents onButtonPressed button: " << EnumToString( button );
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onMuteStateChanged(struct ICOMDeviceEventArgs *args)
{
	VARIANT_BOOL muted = VARIANT_FALSE;
	args->get_mute(&muted);
	std::cout << std::endl << "ICOMDeviceEvents onMuteStateChanged mute: " << ( muted ==  VARIANT_TRUE ? "Muted" : "UnMuted" );
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onAudioStateChanged(struct ICOMDeviceEventArgs *args)
{
	DeviceAudioState deviceState;
	args->get_AudioState(&deviceState);
	std::cout << std::endl << "ICOMDeviceEvents onAudioStateChanged State: " << EnumToString(deviceState);
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onFlashButtonPressed(struct ICOMDeviceEventArgs *args)
{
	std::cout << std::endl << "ICOMDeviceEvents onFlashButtonPressed ";
	return S_OK;
}
STDMETHODIMP DeviceEventSink::onSmartButtonPressed(struct ICOMDeviceEventArgs *args)
{
	std::cout << std::endl << "ICOMDeviceEvents onSmartButtonPressed ";
	return S_OK;
}
STDMETHODIMP DeviceEventExtSink::onBatteryLevelChanged(struct ICOMBatteryLevelEventArgs *args)
{
	DeviceBatteryLevel batteryLevel;
	args->get_BatteryLevel(&batteryLevel);
	std::cout << std::endl << "ICOMDeviceEventsExt onBatteryLevelChanged level: " << EnumToString( batteryLevel );
	return S_OK;
}
STDMETHODIMP DeviceEventExtSink::onHeadsetStateChanged(struct ICOMHeadsetStateEventArgs *args)
{
	short numHs;
	BSTR hsName;
	DeviceProximity devProx;
	DeviceHeadsetState hsState;
	args->get_HeadsetName(&hsName);
	args->get_HeadsetState(&hsState);
	args->get_NumHeadsetsInConference( &numHs );
	args->get_Proximity( &devProx );

	std::string wHsName = "<none>";
	if( hsName != nullptr )
	{
		wHsName = _bstr_t(hsName);
		::SysFreeString( hsName );
	}
	std::cout << std::endl << "ICOMDeviceEventsExt onHeadsetStateChanged name: " << wHsName << " state: " << EnumToString(hsState) << " numInConf: " << numHs << " Proximity: " << EnumToString(devProx);
	if(hsState == HeadsetState_SerialNumber)
	{
		VARIANT serNum;
		args->get_SerialNumber(&serNum);
		std::cout << std::endl << "Headset Serial Number received: " + VariantToString(serNum);
	}

	return S_OK;
}
STDMETHODIMP DeviceEventMobileSink::onPresenceChanged(struct ICOMMobilePresenceEventArgs *args)
{
	BSTR callerId = nullptr;
	DeviceMobileCallState callState;
	args->get_MobileCallState( &callState);

	std::string wCallerId;
	if( callerId != nullptr )
	{
		wCallerId = _bstr_t(callerId);
		::SysFreeString( callerId );
	}
	std::cout << std::endl << "ICOMMobilePresenceEvents onPresenceChanged" << EnumToString(callState) << wCallerId;
	return S_OK;
}
STDMETHODIMP DeviceEventBaseSink::onBaseEventReceived(struct ICOMBaseEventArgs *args)
{
	BaseEventTypeExt eventType;
	args->get_EventType(&eventType);

    std::cout << std::endl << "ICOMBaseEvents onBaseEventReceived eventType: " << EnumToString(eventType);

    DeviceFeatureLock devfetlock;
    while (SUCCEEDED(args->get_FeatureLock(&devfetlock)))
    {
	    std::cout << "  featureLock: " << EnumToString(static_cast<DeviceFeatureLock>(devfetlock)) << ",";
    }
	if(eventType == BaseEventTypeExt_SerialNumber)
	{
		VARIANT serNum;
		args->get_SerialNumber(&serNum);
		std::cout << std::endl << "Base Serial Number received: " + VariantToString(serNum);
	}

	return S_OK;
}

STDMETHODIMP DeviceListenerEventSink::onHeadsetButtonPressed(struct ICOMDeviceListenerEventArgs *args)
{
	DeviceHeadsetButton btn;
	COMDeviceEventType evType;
	args->get_HeadsetButton( &btn );
	args->get_DeviceEventType(&evType);
	
	std::cout << std::endl << "ICOMDeviceListenerEvents onHeadsetButtonPressed hsButton: " << EnumToString(btn) << " devEventType: " << EnumToString( evType );
	return S_OK;
}
STDMETHODIMP DeviceListenerEventSink::onHeadsetStateChanged(struct ICOMDeviceListenerEventArgs *args)
{
	DeviceHeadsetStateChange state;
	COMDeviceEventType evType;
	args->get_HeadsetStateChange( &state );
	args->get_DeviceEventType(&evType);
	
	std::cout << std::endl << "ICOMDeviceListenerEvents onHeadsetStateChanged stateChanged: " << EnumToString(state) << " devEventType: " << EnumToString( evType );
	return S_OK;
}
STDMETHODIMP DeviceListenerEventSink::onBaseButtonPressed(struct ICOMDeviceListenerEventArgs *args)
{
	DeviceBaseButton btn;
	COMDeviceEventType evType;
    SHORT dialedKey;
	args->get_BaseButton(&btn);
	args->get_DeviceEventType(&evType);
	args->get_DialedKey(&dialedKey);

	std::cout << std::endl << "ICOMDeviceListenerEvents onBaseButtonPressed baseButton: " << EnumToString(btn) << " devEventType: " << EnumToString( evType );
    if (BaseButton_DialPad == btn)
    {
        std::cout << ", dialed key = " << dialedKey;
    }
	return S_OK;
}
STDMETHODIMP DeviceListenerEventSink::onBaseStateChanged(struct ICOMDeviceListenerEventArgs *args)
{
	DeviceBaseStateChange state;
	COMDeviceEventType evType;
	args->get_BaseStateChange(&state);
	args->get_DeviceEventType(&evType);

	std::cout << std::endl << "ICOMDeviceListenerEvents onBaseStateChanged stateChanged: " << EnumToString(state) << " devEventType: " << EnumToString( evType );
	return S_OK;
}
STDMETHODIMP DeviceListenerEventSink::onATDStateChanged(struct ICOMDeviceListenerEventArgs *args)
{
	DeviceATDStateChange state;
	COMDeviceEventType evType;
	args->get_ATDStateChange(&state);
	args->get_DeviceEventType(&evType);

	std::cout << std::endl << "ICOMDeviceListenerEvents onATDStateChanged stateChanged: " << EnumToString(state) << " devEventType: " << EnumToString( evType );
	return S_OK;
}