#pragma once

#define PRINT_ERROR(str, res) std::cout << str << "Error Code: " << std::hex << res << std::endl
using namespace ATL;
#include "atlcom.h"
#include <sstream>
#include <string>

// Sink for Session Call Events
class SessionEventSink :
        public CComObjectRootEx<CComSingleThreadModel>,
        public IDispatchImpl<ICOMCallEvents, &__uuidof(ICOMCallEvents), &LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0 >
{
public:	

	BEGIN_COM_MAP(SessionEventSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMCallEvents), IDispatch)
	END_COM_MAP()

	STDMETHOD(onCallRequested)(struct ICOMCallRequestEventArgs *args);
	STDMETHOD(onCallStateChanged)(struct ICOMCallEventArgs *args);	
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
// Sink for Session Manager events
class ManagerEventSink : 
        public CComObjectRootEx<CComSingleThreadModel>,
        public IDispatchImpl<ICOMSessionManagerEvents, &__uuidof(ICOMSessionManagerEvents), &LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0 >
{
public:	

	BEGIN_COM_MAP(ManagerEventSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMSessionManagerEvents), IDispatch)
	END_COM_MAP()

	STDMETHOD(onDeviceStateChanged)( struct ICOMStateDeviceEventArgs *args);
	STDMETHOD(onCallStateChanged)(struct ICOMCallEventArgs *args);	
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
class DeviceEventSink : 
        public CComObjectRootEx<CComSingleThreadModel>,
		public IDispatchImpl<ICOMDeviceEvents,			&__uuidof(ICOMDeviceEvents),		&LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0>
{
public:	

	BEGIN_COM_MAP(DeviceEventSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMDeviceEvents), IDispatch)
	END_COM_MAP()

	// ICOMDeviceEvents callbacks
	STDMETHOD(onDataReceived)(VARIANT *report);
	STDMETHOD(onTalkButtonPressed)(struct ICOMDeviceEventArgs *args);
	STDMETHOD(onButtonPressed)(struct ICOMDeviceEventArgs *args);
	STDMETHOD(onMuteStateChanged)(struct ICOMDeviceEventArgs *args);
	STDMETHOD(onAudioStateChanged)(struct ICOMDeviceEventArgs *args);
	STDMETHOD(onFlashButtonPressed)(struct ICOMDeviceEventArgs *args);
	STDMETHOD(onSmartButtonPressed)(struct ICOMDeviceEventArgs *args);
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
class DeviceEventExtSink : 
        public CComObjectRootEx<CComSingleThreadModel>,
		public IDispatchImpl<ICOMDeviceEventsExt, &__uuidof(ICOMDeviceEventsExt), &LIBID_Spokes3GCOMServerLib , /*wMajor =*/ 3, /*wMinor =*/ 0>
{
public:	

	BEGIN_COM_MAP(DeviceEventExtSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMDeviceEventsExt), IDispatch)
	END_COM_MAP()

	//ICOMDeviceEventsExt callback
	STDMETHOD(onBatteryLevelChanged)(struct ICOMBatteryLevelEventArgs *args);
	STDMETHOD(onHeadsetStateChanged)(struct ICOMHeadsetStateEventArgs *args);
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
class DeviceEventMobileSink: 
        public CComObjectRootEx<CComSingleThreadModel>,
		public IDispatchImpl<ICOMMobilePresenceEvents, &__uuidof(ICOMMobilePresenceEvents), &LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0 >
{
public:	

	BEGIN_COM_MAP(DeviceEventMobileSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMMobilePresenceEvents), IDispatch)
	END_COM_MAP()

	//ICOMMobilePresenceEvents callback
	STDMETHOD(onPresenceChanged)(struct ICOMMobilePresenceEventArgs *args);
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
class DeviceEventBaseSink: 
        public CComObjectRootEx<CComSingleThreadModel>,
		public IDispatchImpl<ICOMBaseEvents, &__uuidof(ICOMBaseEvents), &LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0>
{
public:	

	BEGIN_COM_MAP(DeviceEventBaseSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMBaseEvents), IDispatch)
	END_COM_MAP()

	//ICOMBaseEvents callback
	STDMETHOD(onBaseEventReceived)(struct ICOMBaseEventArgs *args);
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};
// Sink for DeviceListener Events
class DeviceListenerEventSink :
        public CComObjectRootEx<CComSingleThreadModel>,
        public IDispatchImpl<ICOMDeviceListenerEvents, &__uuidof(ICOMDeviceListenerEvents), &LIBID_Spokes3GCOMServerLib, /*wMajor =*/ 3, /*wMinor =*/ 0 >
{
public:	

	BEGIN_COM_MAP(DeviceListenerEventSink)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY_IID(__uuidof(ICOMDeviceListenerEvents), IDispatch)
	END_COM_MAP()

	STDMETHOD(onHeadsetButtonPressed)(struct ICOMDeviceListenerEventArgs *args);
	STDMETHOD(onHeadsetStateChanged)(struct ICOMDeviceListenerEventArgs *args);
	STDMETHOD(onBaseButtonPressed)(struct ICOMDeviceListenerEventArgs *args);
	STDMETHOD(onBaseStateChanged)(struct ICOMDeviceListenerEventArgs *args);
	STDMETHOD(onATDStateChanged)(struct ICOMDeviceListenerEventArgs *args);	
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
};

// wrapper to Advise/Unadvise on Session Manager events
class SessionManagerSink
{
public:
	SessionManagerSink(ICOMSessionManagerPtr sessionManager)
	{
		m_sessionManager = sessionManager;
		CComObject<ManagerEventSink>::CreateInstance(&m_sink);
		if(m_sessionManager != nullptr && m_sink != nullptr )
		{
			m_sink->AddRef();
			HRESULT hr = AtlAdvise(m_sessionManager, m_sink, __uuidof(ICOMSessionManagerEvents), &m_cookie);
			if( FAILED( hr ) ) PRINT_ERROR( "Advise Session Manager events Error" , hr);
		}
	}
	~SessionManagerSink()
	{
		HRESULT hr = AtlUnadvise(m_sessionManager, __uuidof(ICOMSessionManagerEvents), m_cookie);
		if( FAILED( hr) ) PRINT_ERROR( "Unadvise Session Manager events Error" , hr );
		m_sink->Release();
	}
private:
	CComObject<ManagerEventSink> *m_sink;
	ICOMSessionManagerPtr m_sessionManager;
	DWORD m_cookie;
};
// wrapper to Advise/Unadvise on Session Call events
class SessionSink
{
public:
	SessionSink(ICOMSessionPtr session)
	{
		if(session != nullptr )
		{
			m_session = session;
			if( SUCCEEDED( CComObject<SessionEventSink>::CreateInstance(&m_sink) ) && m_sink != nullptr )
			{
				m_sink->AddRef();
				HRESULT hr = AtlAdvise(m_session, m_sink, __uuidof(ICOMCallEvents), &m_cookie);
				if( FAILED( hr ) ) PRINT_ERROR( "Advise Session Call event Error" , hr);
			}
		}
	}
	~SessionSink()
	{
		HRESULT hr = AtlUnadvise(m_session, __uuidof(ICOMCallEvents), m_cookie);
		if( FAILED( hr ) ) PRINT_ERROR( "Unadvise Session Call event Error" , hr);
		m_sink->Release();
	}
private:
	CComObject<SessionEventSink> *m_sink;
	ICOMSessionPtr m_session;
	DWORD m_cookie;
};
// wrapper to Advise/Unadvise on Device Listener events
class DeviceListenerSink
{
public:
	DeviceListenerSink( ICOMSessionPtr session)
	{
		if( session != nullptr )
		{
			ICOMDevicePtr device;
			if( SUCCEEDED( session->GetActiveDevice(&device) ) && device != nullptr )
			{
				if( SUCCEEDED( device->get_DeviceListener(&m_deviceListener) ) && m_deviceListener != nullptr )
				{
					if( SUCCEEDED( CComObject<DeviceListenerEventSink>::CreateInstance(&m_sink) ) && m_sink != nullptr )
					{
						m_sink->AddRef();
						HRESULT hr = AtlAdvise(m_deviceListener, m_sink, __uuidof(ICOMDeviceListenerEvents), &m_cookie);
						if( FAILED( hr ) ) PRINT_ERROR( "Advise Device Listener event Error: " , hr);
					}
				}
			}
		}
	}
	~DeviceListenerSink()
	{
		HRESULT hr = AtlUnadvise(m_deviceListener, __uuidof(ICOMDeviceListenerEvents), m_cookie);
		if( FAILED( hr ) ) PRINT_ERROR( "Unadvise Device Listener event Error: " , hr);
		m_sink->Release();
	}
private:
	CComObject<DeviceListenerEventSink> *m_sink;
	ICOMDeviceListenerPtr m_deviceListener;
	DWORD m_cookie;
};
// wrapper to Advise/Unadvise on Device events
class DeviceSink
{
public:
	DeviceSink( ICOMSessionPtr session)
	{
		if( session != nullptr )
		{
			if( SUCCEEDED( session->GetActiveDevice(&m_device) ) && m_device != nullptr )
			{
				// hook to device listener event
				m_devListSink = new DeviceListenerSink( session );

				if( SUCCEEDED( CComObject<DeviceEventSink>::CreateInstance(&m_sink1) ) && m_sink1 != nullptr &&
					SUCCEEDED( CComObject<DeviceEventExtSink>::CreateInstance(&m_sink2) ) && m_sink2 != nullptr &&
					SUCCEEDED( CComObject<DeviceEventMobileSink>::CreateInstance(&m_sink3) ) && m_sink3 != nullptr &&
					SUCCEEDED( CComObject<DeviceEventBaseSink>::CreateInstance(&m_sink4) ) && m_sink4 != nullptr )
				{
					AtlAdvise(m_device, m_sink1, __uuidof(ICOMDeviceEvents), &m_cookie1);
					AtlAdvise(m_device, m_sink2, __uuidof(ICOMDeviceEventsExt), &m_cookie2);
					AtlAdvise(m_device, m_sink3, __uuidof(ICOMMobilePresenceEvents), &m_cookie3);
					AtlAdvise(m_device, m_sink4, __uuidof(ICOMBaseEvents), &m_cookie4);
				}
			}
		}
	}
	~DeviceSink()
	{
		delete m_devListSink;

		AtlUnadvise(m_device, __uuidof(ICOMDeviceEvents), m_cookie1);
		AtlUnadvise(m_device, __uuidof(ICOMDeviceEventsExt), m_cookie2);
		AtlUnadvise(m_device, __uuidof(ICOMMobilePresenceEvents), m_cookie3);
		AtlUnadvise(m_device, __uuidof(ICOMBaseEvents), m_cookie4);
	}
private:
	DeviceListenerSink* m_devListSink;
	CComObject<DeviceEventSink> *m_sink1;
	CComObject<DeviceEventExtSink> *m_sink2;
	CComObject<DeviceEventMobileSink> *m_sink3;
	CComObject<DeviceEventBaseSink> *m_sink4;
	ICOMDevicePtr m_device;
	DWORD m_cookie1;
	DWORD m_cookie2;
	DWORD m_cookie3;
	DWORD m_cookie4;
};

