#pragma once
#include <string>
#include <iostream>
#include <algorithm>
#include <stdio.h>
#include <tchar.h>
#include "atlsafe.h"
#include "CallObject.h"
#include "Command.h"

class SessionManagerCommand : public Command
{
public:
	void Execute(ICOMSessionPtr session, ICOMSessionManagerPtr sessionManager)
	{
		HRESULT res = S_OK;
		BSTR valBstr = nullptr;
		BSTR valBstr1 = nullptr;
		VARIANT_BOOL valBool;
		short valShort = 0;
		long valLong = 0;

		do
		{
			std::cout << "ISessionManager > ";

			// get user unput
			std::string input;
			std::getline(std::cin, input);
			std::transform(input.begin(), input.end(), input.begin(), ::tolower);
			if( input == "exit") break;

			// execute command
			if	   ( input == "userpreferences" ) 
			{
				ICOMUserPreferencePtr userPref;
				if( SUCCEEDED(res = sessionManager->get_UserPreference(&userPref) ) )
				{
					do
					{
						std::cout << "ISessionManager > IUserPreferences > ";

						// get user unput
						std::getline(std::cin, input);
						std::transform(input.begin(), input.end(), input.begin(), ::tolower);
						if( input == "exit") break;
						 
						if		( input == "getmediaplayeractionincomingcall" ){ userPref->get_MediaPlayerActionIncomingCall( &valBstr ); PrintReturn( valBstr ); }
						else if	( input == "getmediaplayeractionendedcall" ){ userPref->get_MediaPlayerActionEndedCall( &valBstr ); PrintReturn( valBstr ); }
						else if	( input == "getringpcandhs" ){ userPref->get_RingPCAndHS( &valBool ); PrintReturn( valBool ); }
						else if	( input == "getdefaultsoftphone" ){ userPref->get_DefaultSoftphone( &valBstr ); PrintReturn( valBstr ); }
						else if	( input == "getescalatetovoicesoftphone" ){ userPref->get_EscalateToVoiceSoftPhone( &valBstr ); PrintReturn( valBstr ); }
						else if	( input == "getkeeplinkup" ){ userPref->get_KeepLinkUp( &valBool ); PrintReturn( valBool ); }
						else if	( input == "getautopresence" ){ userPref->get_AutoPresence( &valBool ); PrintReturn( valBool ); }
						else if	( input == "getdonaction" ){ userPref->get_DonAction( &valBstr ); PrintReturn( valBstr ); }
						else if	( input == "getdoffaction" ){ userPref->get_DoffAction( &valBstr ); PrintReturn( valBstr ); }

						else if	( input == "putmediaplayeractionincomingcall" ){ userPref->put_MediaPlayerActionIncomingCall( valBstr = _bstr_t(  GetString("Enter media action on incoming call: " ).c_str() ).Detach() );  }
						else if	( input == "putmediaplayeractionendedcall" ){ userPref->put_MediaPlayerActionEndedCall( valBstr = _bstr_t(  GetString("Enter media action on ended call: " ).c_str() ).Detach()  ); }
						else if	( input == "putpingpcandhson" ){ userPref->put_RingPCAndHS( VARIANT_TRUE );}
						else if	( input == "putpingpcandhsoff" ){ userPref->put_RingPCAndHS( VARIANT_FALSE );}
						else if	( input == "putdefaultsoftphone" ){ userPref->put_DefaultSoftphone( valBstr = _bstr_t(  GetString("Set default SoftPhone: " ).c_str() ).Detach() );  }
						else if	( input == "putescalatetovoicesoftphone" ){ userPref->put_EscalateToVoiceSoftPhone( valBstr = _bstr_t(  GetString("Set escalate voice SoftPhone: " ).c_str() ).Detach() ); }
						else if	( input == "putkeeplinkupon" ){ userPref->put_KeepLinkUp( VARIANT_TRUE ); }
						else if	( input == "putkeeplinkupoff" ){ userPref->put_KeepLinkUp( VARIANT_FALSE ); }
						else if	( input == "putautopresenceon" ){ userPref->put_AutoPresence( VARIANT_TRUE );  }
						else if	( input == "putautopresenceoff" ){ userPref->put_AutoPresence( VARIANT_FALSE ); }
						else if	( input == "putdonaction" ){ userPref->put_DonAction( valBstr = _bstr_t(  GetString("Set DON action: " ).c_str() ).Detach() ); }
						else if	( input == "putdoffaction" ){ userPref->put_DoffAction( valBstr = _bstr_t(  GetString("Set DOFF action: " ).c_str() ).Detach() );}
						else continue;

						std::cout << std::endl;

					}while(true);
				}
				else
					PRINT_STATUS( res );
			}
			else if( input == "printcalls" )
			{
				ICOMCallManagerStatePtr callState;
				if( SUCCEEDED(res = sessionManager->get_CallManagerState(&callState) ) )
				{
					printCallManagerState(callState);
				}
			}
			else if( input == "help")
			{
				std::cout << "userpreferences   " << "sessionManager->getUserPreference"  << std::endl;
				std::cout << "printcalls	    " << "display state of calls in Session Manager"  << std::endl;
				continue;
			}
			else { std::cout << "type help for list of commands" << std::endl; continue; }
			// Print command execution status
			PRINT_STATUS( res );

		} while ( true );

		if( valBstr ) ::SysFreeString( valBstr );
		delete this;
	}

	void printCallManagerState(ICOMCallManagerStatePtr callState)
	{
		if(callState != nullptr)
		{
			VARIANT_BOOL hasActive = false;
			short numHold = 0;
			SAFEARRAY *calls;

			callState->get_HasActiveCall(&hasActive);
			callState->get_NumberOfHoldCalls( &numHold );
			callState->get_Calls( &calls );

			std::cout << "Active Call:             " << ( hasActive == VARIANT_TRUE ? "Yes" : "No" ) << std::endl;
			std::cout << "Number of calls on hold: " <<  numHold << std::endl;

			long arrStartIndex, arrEndIndex;
			if (SUCCEEDED ( SafeArrayGetLBound(calls, 1, &arrStartIndex) ) &&
				SUCCEEDED ( SafeArrayGetUBound(calls, 1, &arrEndIndex) ) )
			{
				for (long currIndex = arrStartIndex; currIndex <= arrEndIndex; currIndex++)
				{
					CComPtr<ICOMCallInfo> callInfo;
					VARIANT var;
					
					if (SUCCEEDED(SafeArrayGetElement(calls, &currIndex, &var)))
					{
						if (var.vt == VT_DISPATCH && SUCCEEDED(var.pdispVal->QueryInterface(__uuidof(ICOMCallInfo), (void **) &callInfo)))
						{
							long callId;
							BSTR sessionId, source;
							VARIANT_BOOL callActive;

							callInfo->get_CallId(&callId);
							callInfo->get_CallSource(&source);
							callInfo->get_SessionId(&sessionId);
							callInfo->get_CallActive(&callActive);

							_bstr_t sessId(sessionId), src(source);
																				
							std::cout << "CallId: " << callId << ", IsActive: " << ((callActive==VARIANT_TRUE) ? "TRUE" : "FALSE") << ", Source: \"" << src << "\", sessionID: \"" << sessId << "\"" << std::endl;
						}
					}
				}
			}
		}
	}
};
