// SpokesSamples.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

// We need this as project is created as Win console app, and we are instantiating ATL COM objects that are used as Event sink's
CComModule _Module;
extern __declspec(selectany) CAtlModule* _pAtlModule=&_Module;


#define PRINT_STATUS(res) SUCCEEDED( res ) ? (std::cout << "Status Success" << std::endl) : std::cout << "Failed " << std::hex << res << std::endl
#define PRINT_ERROR(str, res) std::cout << str << "Error Code: " << std::hex << res << std::endl

#include "EventSink.h"
#include "CallCommand.h"
#include "DeviceCommand.h"
#include "DeviceListenerCommand.h"
#include "SessionManagerCommand.h"
#include "HelpCommand.h"
extern DeviceSink* gDeviceSink;

// create object that will deal  
Command* GetCommand(std::string cmd)
{
	std::transform(cmd.begin(), cmd.end(), cmd.begin(), ::tolower);
	if( cmd == "icallcommand" )		return new CallCommand();
	if( cmd == "ihostcommand")		return new DeviceCommand();
	if( cmd == "idevicelistener")	return new DeviceListener();
	if( cmd == "isessionmanager")	return new SessionManagerCommand();
	if( cmd == "help" )				return new HelpCommand();
	else return nullptr;
}

// sample entry point
int _tmain(int argc, _TCHAR* argv[])
{
	// Initialize COM
	::CoInitializeEx(NULL, COINIT_MULTITHREADED);
	{
		// Create Spokes session manager object
		ICOMSessionManagerPtr sessionManager;
		if( SUCCEEDED ( sessionManager.CreateInstance( CLSID_COMSessionManager ) ) )
		{
			// Hook into Session Manager events
			SessionManagerSink sessionManagerSink( sessionManager );

			// Register new session in Spokes.
			ICOMSessionPtr session;
			if( SUCCEEDED (sessionManager->Register( _bstr_t("COM Plugin") , &session) ) )
			{
				// Hook into Session  events
				SessionSink sessionSink( session );
				
				// Hook into Device events. When device is detached, this object need to be recreated.
				gDeviceSink = new DeviceSink( session );

                ICOMUserPreference *userPref;
                sessionManager->get_UserPreference(&userPref);
                userPref->put_DefaultSoftphone(_bstr_t("COM Plugin"));

				std::string input;
				do
				{
					// Base on user input, create proper object that will provide set of specific commands
					std::cout << " > ";
					std::getline(std::cin, input);
					
					Command* command = GetCommand(input);
					if( command != nullptr ) command->Execute( session, sessionManager);
					
				} while (input != "exit" );

				// do cleanup
				sessionManager->Unregister( session );
			}
		}
	}
	// uninitialize COM
	::CoUninitialize(); 

	return 0;
}

