﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Interop.Plantronics;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace SpokesSDKCOMNETSample
{
    class CallCommand : Command
    {

        private static string MakeCall = "makecall";
        private static string HoldCall = "holdcall";
        private static string EndCall = "endcall";
        private static string AnsweredCall = "answeredcall";
        private static string ResumeCall = "resumecall";
        private static string Incoming = "incoming";
        private static string Outgoing = "outgoing";

        private static List<string> supportedCommands = new List<string>(new string[] { MakeCall, HoldCall, EndCall, AnsweredCall, ResumeCall, Incoming, Outgoing });

        override protected void DoExecute(string command, COMSession session)
        {
            COMCallCommand callCommand = session.GetCallCommand();
            if (callCommand != null)
            {
                try
                {
                    if (command == Incoming)
                        callCommand.IncomingCall(GetCall(), CreateContact(), CallRingTone.RingTone_Unknown, CallAudioRoute.AudioRoute_ToHeadset);
                    else if (command == Outgoing)
                        callCommand.OutgoingCall(GetCall(), CreateContact(), CallAudioRoute.AudioRoute_ToSpeaker);
                    else if (command == AnsweredCall)
                        callCommand.AnsweredCall(GetCall());
                    else if (command == HoldCall)
                        callCommand.HoldCall(GetCall());
                    else if (command == ResumeCall)
                        callCommand.ResumeCall(GetCall());
                    else if (command == EndCall)
                        callCommand.TerminateCall(GetCall());
                    else if (command == MakeCall)
                        callCommand.MakeCall(CreateContact());
                }
                catch (COMException ex)
                {
                    Console.WriteLine("COM exception: " + ex.Message);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Exception: " + ex.Message);
                }
            }
            else
                Console.WriteLine("Could not retrieve call command object");
        }

        override public ReadOnlyCollection<string> GetSupportedCommands()
        {
            return supportedCommands.AsReadOnly();
        }

        private COMCall GetCall()
        {
            COMCall call = new COMCall();
            call.Id = GetCallId();
            return call;
        }

        private COMContact CreateContact()
        {
            Contact contact = new Contact();
            contact.Name = GetRandomContactName();
            return contact;
        }

        private int GetCallId()
        {
            return 42;
        }

        private string GetRandomContactName()
        {
            return "Random Contact";
        }
    }
    
    class Contact : COMContact
    {
        public string Email { get; set; }
        public string FriendlyName { get; set; }
        public string HomePhone { get; set; }
        public int Id { get; set; }
        public string MobilePhone { get; set; }
        public string Name { get; set; }
        public string Phone { get; set; }
        public string SipUri { get; set; }
        public string WorkPhone { get; set; }
    }
}
