﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Interop.Plantronics;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace SpokesSDKCOMNETSample
{
    class DeviceCommand : Command
    {
        private static string RingOn = "ringon";
        private static string RingOff = "ringoff";
        private static string AudioOn = "audioon";
        private static string AudioOff = "audiooff";
        private static string GetHeadsetType = "gethstype";
        private static string GetHeadsetState = "gethsstate";

        private static List<string> supportedCommands = new List<string>(new string[] { RingOn, RingOff, AudioOn, AudioOff, GetHeadsetType, GetHeadsetState });

        override protected void DoExecute(string command, COMSession session)
        {
            COMDevice device = session.GetActiveDevice();
            if (device != null)
            {
                COMHostCommand hostCommand = device.HostCommand;
                if (hostCommand != null)
                {
                    try
                    {
                        if (command == RingOn)
                            hostCommand.SetRing = true;
                        else if (command == RingOff)
                            hostCommand.SetRing = false;
                        else if (command == AudioOn)
                            hostCommand.AudioState = DeviceAudioState.AudioState_MonoOn;
                        else if (command == AudioOff)
                            hostCommand.AudioState = DeviceAudioState.AudioState_MonoOff;
                        else
                        {
                            ICOMHostCommandExt hsExt = device.HostCommand as ICOMHostCommandExt;
                            if (hsExt != null)
                            {
                                if (command == GetHeadsetType)
                                    Console.WriteLine("HeadsetType: {0}", hsExt.HeadsetType.ToString());
                                else if (command == GetHeadsetState)
                                    Console.WriteLine("HeadsetState: {0}", hsExt.HeadsetState.ToString());
                            }
                            else
                                Console.WriteLine("Could not retrieve HostCommandExt object while executing {0} command", command);
                        }
                    }
                    catch (COMException ex)
                    {
                        Console.WriteLine("COM exception: " + ex.Message);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine("Exception: " + ex.Message);
                    }
                }
                else
                    Console.WriteLine("Could not retrieve host command object");
            }
            else
                Console.WriteLine("Could not retrieve device object");
        }

        override public ReadOnlyCollection<string> GetSupportedCommands()
        {
            return supportedCommands.AsReadOnly();
        }
    }
}
