﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Interop.Plantronics;
using System.Runtime.InteropServices;

namespace SpokesSDKCOMNETSample
{
    class SessionEventsListener : EventListener
    {
        private COMSession session = null;
        private ICOMCallEvents_Event callEvents = null;

        public SessionEventsListener(COMSession session)
        {
            this.session = session;

            callEvents = session as ICOMCallEvents_Event;
            if (callEvents != null)
            {
                callEvents.onCallRequested += onCallRequested;
                callEvents.onCallStateChanged += onCallStateChanged;

                PrintSessionInfo();

                Console.WriteLine("Successfully hooked to Session events");
            }
            else
                Console.WriteLine("Unable to hook to Session events");
        }

        ~SessionEventsListener()
        {
            if (callEvents != null)
            {
                callEvents.onCallRequested -= onCallRequested;
                callEvents.onCallStateChanged -= onCallStateChanged;

                Marshal.ReleaseComObject(callEvents);
                callEvents = null;
            }

            //no session object disposal, because it came from outside
            //he who created session should also dispose it
        }

        private void PrintSessionInfo()
        {
            if (session == null)
                return;

            Console.WriteLine("********************* Session Information ******************** ");
            Console.WriteLine("Plugin Name:    " + session.PluginName);
            Console.WriteLine("Session Id:     " + session.SessionId);
            Console.WriteLine("************************************************************* ");
        }

        void onCallRequested(COMCallRequestEventArgs callRequestedEventArgs)
        {
            Console.WriteLine("Received call request from: " + callRequestedEventArgs.contact.Name + " id: " + callRequestedEventArgs.contact.Id);
        }
     
        void onCallStateChanged(COMCallEventArgs callEventArgs)
        {
            Console.WriteLine("Call state changed: " + callEventArgs.CallState.ToString());
        }
    }
}
