﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Interop.Plantronics;
using System.Runtime.InteropServices;
using SpokesSDKCOMNETSample.Commands;

namespace SpokesSDKCOMNETSample
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                ICOMSessionManager sessionManager = new COMSessionManager();
                Console.WriteLine("Successfully created SessionManager");
                COMSession session;
                sessionManager.Register("SDK .NET COM sample", out session);

                //hook to SessionManager, Session and Device events
                SessionManagerEventsListener sessionManagerEventListener = new SessionManagerEventsListener(sessionManager);
                SessionEventsListener sessionEventsListener = new SessionEventsListener(session);
                DeviceEventsListener deviceEventsListener = new DeviceEventsListener(session);

                ExecuteCommands(session);

                //cleanup 
                sessionManager.Unregister(session);
                Marshal.ReleaseComObject(session);
                Marshal.ReleaseComObject(sessionManager);
            }
            catch (System.Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
        }

        private static void ExecuteCommands(COMSession session)
        {
            //instantiate commands
            ICommand callCommand = CommandFactory.GetCommand(CommandType.CallCommand);
            ICommand deviceCommand = CommandFactory.GetCommand(CommandType.DeviceCommand);
            ICommand deviceListenerCommand = CommandFactory.GetCommand(CommandType.DeviceListenerCommand);

            IDictionary<string, ICommand> commandMap = new Dictionary<string, ICommand>(); 
            //no duplicated commands allowed!
            PrepareCommandMap(commandMap, callCommand, deviceCommand, deviceListenerCommand);

            for (;;)
            {
                Console.Write(">>");
                string command = Console.ReadLine().ToLowerInvariant();
                ICommand commandObject;

                if (commandMap.TryGetValue(command, out commandObject))
                    commandObject.Execute(command, session);
                else if (command == "help")
                {
                    Console.WriteLine("Supported commands");
                    Console.WriteLine("ringon:        calls IHostCommand::Ring()");
                    Console.WriteLine("ringoff:       calls IHostCommand::Ring()");
                    Console.WriteLine("audioon:       calls IHostCommand::put_AudioState()");
                    Console.WriteLine("audiooff:      calls IHostCommand::put_AudioState()");
                    Console.WriteLine("gethstype:     calls IHostCommandExt::get_HeadsetType()");
                    Console.WriteLine("gethsstate:    calls IHostCommandExt::get_HeadsetState()");
                    Console.WriteLine("muteon:        calls IDeviceListener::put_Mute()");
                    Console.WriteLine("muteoff:       calls IDeviceListener::put_Mute()");
                    Console.WriteLine("makecall:      calls ICallCommand::MakeCall)");
                    Console.WriteLine("holdcall:      calls ICallCommand::HoldCall()");
                    Console.WriteLine("endcall:       calls ICallCommand::TerminateCall()");
                    Console.WriteLine("answeredcall:  calls ICallCommand::AnsweredCall()");
                    Console.WriteLine("resumecall:    calls ICallCommand::ResumeCall()");
                    Console.WriteLine("incoming:      calls ICallCommand::IncomingCall()");
                    Console.WriteLine("outgoing:      calls ICallCommand::OutgoingCall()");
                    Console.WriteLine("exit: close application");
                }
                else if (command == "exit")
                {
                    break;
                }
                else
                    Console.WriteLine("Unknown command \"{0}\". Type \"help\" for command list", command);
            }
        }

        private static void PrepareCommandMap(IDictionary<string, ICommand> commandMap, params ICommand[] commands)
        {
            foreach (ICommand commandObject in commands)
            {
                foreach (string command in commandObject.GetSupportedCommands())
                {
                    commandMap.Add(command, commandObject);
                }
            }
        }
    }
}
